
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "Character.hpp"

// --------------------------------------------------------

Character::Character(Environment *env, MapManager* aMap,SDL_Surface *aSprite,int aSpeed)
{
	environment = env;
	map = aMap;
	direction = dirNone;
	numSprite = 0;
	speed = aSpeed;
	sprite = aSprite;
	moveWaiting = false;
	fallingDown = false;
}

// --------------------------------------------------------

Character::~Character()
{
}

// --------------------------------------------------------

void Character::evolve(void)
{
	if (fallingDown)
		fallDown();
	else
	{
		switch (direction)
		{
			case dirRight:
				offset.x += speed;
				break;
			case dirLeft:
				offset.x -= speed;
				break;
			case dirUp:
				offset.y -= speed;
				break;
			case dirDown:
				offset.y += speed;
				break;
			//case dirNone:
			//	break;
		}
		numSprite = (numSprite+1)%_NB_TILE_SPRITE;

		// has finish it's current move ??
		if (offset.x >= _MAP_BLOCK_SIZE)	
		{
			offset.x = 0;
			setPosition(position.x + 1, position.y);
		}
		else if (offset.x <= -_MAP_BLOCK_SIZE)	
		{
			offset.x = 0;
			setPosition(position.x - 1, position.y);
		}
		if (offset.y >= _MAP_BLOCK_SIZE)	
		{
			offset.y = 0;
			setPosition(position.x, position.y + 1);
		}
		else if (offset.y <= -_MAP_BLOCK_SIZE)	
		{
			offset.y = 0;
			setPosition(position.x, position.y - 1);
		}

		if ((offset.x == 0)	&& (offset.y == 0))
		{
			moveWaiting = false;
		}
		checkCurrentPosition();
	}
}

// --------------------------------------------------------

void Character::checkCurrentPosition(void)
{
	// the character exits the screen ??
	if (!map->isInMap(position)) 
		exitScreen();

	// the character falls down ??
	if ((map->isBlank(position.x,position.y+1)) && (map->isBlank(position)))
	{
		fallingDown = true;
		moveWaiting = true;
	}
}

// --------------------------------------------------------

bool Character::moveLeft(void)
{
	
	if(!isMoveWaiting())
	{
		GLPoint nextPosition(position.x - 1, position.y);

		// test if Character will go outside of the map
		if(!map->isInMap(nextPosition))
    		return false;

		if(!map->isGround(nextPosition))
		{
			direction = dirLeft;
  			moveWaiting = true;
			return true;
		}
	}
	return false;
}

// --------------------------------------------------------

bool Character::moveRight(void)
{
	if(!isMoveWaiting())
	{
		GLPoint nextPosition(position.x + 1, position.y);

		// test if Character will go outside of the map
		if(!map->isInMap(nextPosition))
    		return false;

		if(!map->isGround(nextPosition))
		{
			direction = dirRight;
			moveWaiting = true;
			return true;
		}
	}
	return false;
}
	
// --------------------------------------------------------

bool Character::moveUp(void)
{
	if(!isMoveWaiting())
	{
		GLPoint nextPosition(position.x, position.y - 1);

		// test if Character will go outside of the map or into ground
		if((!map->isInMap(nextPosition)) || (map->isGround(nextPosition)))
    		return false;

		if(map->isScale(position))
		{
			direction = dirUp;
  			moveWaiting = true;
			return true;
		}
	}
	return false;
}

// --------------------------------------------------------

bool Character::moveDown(void)
{
	if(!isMoveWaiting())
	{
		GLPoint nextPosition(position.x, position.y + 1);

		// test if Character will go outside of the map or into ground
		if((!map->isInMap(nextPosition)) || (map->isGround(nextPosition)))
    		return false;

		if((map->isScale(nextPosition)) || (map->isScale(position)))
		{
			direction = dirDown;
  			moveWaiting = true;
			return true;
		}
	}
	return false;
}

// --------------------------------------------------------

void Character::fallDown(void)
{
	// has finish it's current move ???
	if (offset.y >= _MAP_BLOCK_SIZE)	
	{	
		offset.y -= _MAP_BLOCK_SIZE;
		setPosition(position.x, position.y + 1);

		checkCurrentPosition();

		if(map->isGround(position.x, position.y+1) || map->isScale(position.x,position.y+1))
		{
			fallingDown = false;
			moveWaiting = false;
		}
	}

	if (fallingDown)
		offset.y += speed;
}

// --------------------------------------------------------

/**
 *
 */
void Character::exitScreen(void)
{
	initialize();
}

// --------------------------------------------------------

void Character::crush(void)
{
	initialize();
}

// --------------------------------------------------------

void Character::display(DisplayManager *aDisplay, const GLPoint& mapOffset)
{
	aDisplay->addSprite(getSprite(), getPixelPosition().x - mapOffset.x, getPixelPosition().y - mapOffset.y, getNumSprite(), getDirection());
}

// --------------------------------------------------------

GLPoint Character::getPixelPosition(void) const
{
	return (position * _MAP_BLOCK_SIZE) + offset;
}

// --------------------------------------------------------

void Character::initialize(void)
{
	moveWaiting = false;
	fallingDown = false;
	setPosition(0, 0);
	offset.set(0,0);
	direction = dirNone;
	numSprite = 0;
}

// --------------------------------------------------------

void Character::setPosition(int x, int y)
{
	position.x = x; 
	position.y = y;
}

// --------------------------------------------------------

void Character::setOffset(int x, int y)
{
	offset.x = x; 
	offset.y = y;
}

// --------------------------------------------------------
