
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _CHARACTER_
#define _CHARACTER_

#include "DisplayManager.hpp"
#include "MapManager.hpp"
#include "Environment.hpp"

#include "graphlib/GLPoint.hpp"

// --------------------------------------------------------

#define _NB_TILE_SPRITE		5

// --------------------------------------------------------

typedef enum {dirNone = 0, dirRight = 0, dirLeft, dirUp, dirDown} EDirection;

// --------------------------------------------------------

class Character
{
	protected :
		// if the perso has to move
		bool moveWaiting;
		// if perso is falling down
		bool fallingDown;
		// block based position in map.
		GLPoint position;
		// number of pixels that hero walked. must be 32 to stop moving (used by Environment)
		GLPoint offset;
		EDirection direction;
		int speed;
		int numSprite;

		SDL_Surface *sprite;
		MapManager *map;
		Environment *environment;

	public :
		Character(Environment *env, MapManager* map,SDL_Surface *sprite,int speed);
		virtual ~Character();

		void display(DisplayManager *aDisplay, const GLPoint& mapOffset);
		// move by pixels (speed = nb pixels move)
		void evolve(void);	
		bool moveLeft(void);
		bool moveRight(void);
		bool moveUp(void);
		bool moveDown(void);
		virtual void setPosition(int x, int y);
		void setOffset(int x, int y);
		bool isHere(int x, int y);
		bool isMoveWaiting(void);
  		EDirection getDirection(void);
  		int getSpeed(void);
		const GLPoint& getPosition(void) const;
		GLPoint getPixelPosition(void) const;
		int getNumSprite(void);
		SDL_Surface *getSprite(void);

		virtual void checkCurrentPosition(void);
		virtual void fallDown(void);
		virtual void exitScreen(void);
		virtual void crush(void);
		virtual void initialize(void);
};

// --------------------------------------------------------
// inline methods

inline
bool Character::isHere(int x, int y)
{
	return ((x == position.x) && (y == position.y));
}

// --------------------------------------------------------

inline
bool Character::isMoveWaiting(void)
{
//	return (moveWaiting || fallingDown);
	return moveWaiting;
}

// --------------------------------------------------------

inline
EDirection Character::getDirection(void)
{
	return direction;
}

// --------------------------------------------------------

inline
int Character::getSpeed(void)
{
	return speed;
}

// --------------------------------------------------------

inline
const GLPoint& Character::getPosition(void) const
{
	return position;
}

// --------------------------------------------------------

inline
int Character::getNumSprite(void)
{
	return numSprite;
}

// --------------------------------------------------------

inline
SDL_Surface *Character::getSprite(void)
{
	return sprite;
}

// --------------------------------------------------------

#endif	// _CHARACTER_
