
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _DISPLAYMANAGER_
#define _DISPLAYMANAGER_

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>

#include "graphlib/GLPoint.hpp"
#include "graphlib/GLFont.hpp"

// --------------------------------------------------------

#define _TAILLESPRITEPIX	 32		// size of the sprite in pixels
#define _REFRESHZONE		  5		// max move in pixels
#define _APPL_TITLE			"Bibom"

// --------------------------------------------------------

typedef enum {mcNew, mcLoad, mcOptions, mcCredits, mcQuit} EMenuChoice;

// --------------------------------------------------------

class DisplayManager
{
	private :
		SDL_Surface *screen;
		SDL_Surface *background;
		SDL_Surface *controlKeys;
		SDL_Surface *controlLifes;
		char ** map;
		int spritesCount;
		SDL_Rect* refreshRects;

		GLPoint screenSize;

		GLFont *font;
	public :
		DisplayManager(void);
		virtual ~DisplayManager();

		void initDisplay(void);
		void closeDisplay(void);
		void initView(void);

		const GLPoint& getSize(void) const;

		void createScreen(int width, int height, int depth, bool fullScreen);
		void createBackground(char *name);

		void addSprite(SDL_Surface* sprite, int x, int y, int column, int row); 
		//numsprite is the sprite number
		void addMapSprite(SDL_Surface* sprite, int x, int y, int column); 
		
		void refreshScreen(void);
		
		void displayMenu(EMenuChoice item);
		void displayLevelIntro(const char *name);
		void displayControlBar(int keysCount, int lifeCount);
		void displayGameOver(void);
		void displayCredits(void);
		
};

// --------------------------------------------------------
// inline methods

inline
const GLPoint& DisplayManager::getSize(void) const
{
	return screenSize;
}

// --------------------------------------------------------

#endif	// _DISPLAYMANAGER_
