
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _ENVIRONMENT_
#define _ENVIRONMENT_

#include "toollib/TLArray.hpp"
#include "graphlib/GLPoint.hpp"

#include <SDL.h>

// --------------------------------------------------------

class Hero;
class Enemy;
class MapManager;
class DisplayManager;

// --------------------------------------------------------

#define _MAXENEMY	3

#define _TIME_RECOVERY_BLOCK	400

#define _MOTIONLESS_SCREEN_MARGIN_W	100
#define _MOTIONLESS_SCREEN_MARGIN_H	100

#define _MAP_SCROLLING_SPEED	4

// --------------------------------------------------------

typedef struct
{
	GLPoint position;
	int timeElapsed;
} DestroyedBlock;

// --------------------------------------------------------

class Environment
{
	private:
		SDL_Surface *mapFloor;
		SDL_Surface *mapScale;
		SDL_Surface *mapDoorE;
		SDL_Surface *mapDoorS;
		SDL_Surface *mapKey;
		SDL_Surface *mapScaleDown;
		SDL_Surface *mapScaleUp;

		DisplayManager *display;
		MapManager *map;
		Hero* hero;

		// array of enemies
		Enemy** enemy;
		TLArray destroyedBlocks;

		// scrolling of map...
		GLPoint mapSize;
		GLPoint mapOffset;
		GLPoint newMapOffset;
		SDL_Rect motionLessScreen;

	public:
		Environment(DisplayManager* display, MapManager* map);
		virtual ~Environment();

		const GLPoint& getMapOffset(void) const;
		void setMapOffset(const GLPoint& offset);
		void setMapOffset(int x, int y);
		const SDL_Rect& getMotionLessZone(void) const;
		const GLPoint& getScreenSize(void) const;

		Hero *getHero(void);
		// prepare enviro for new level
		void newLevel(void);
		// refresh display and objects
		void refreshView(void);
		// return 0 if not, 1 if yes, 2 if hero dead
		int levelFinished(void);
		// Making the perso evolve with their speed
		void evolve(void);

		void destroyBlock(int x, int y);
		void destroyBlock(const GLPoint& position);
		void updateTimeDestroyedBlock(void);

	private:
		void createHero(void);
		void createEnemies(void);
		void displayMap(void);
		bool makeEnvironmentEvolve(void);
		bool makeCharactersEvolve(void);
};

// --------------------------------------------------------
// inline methods

inline
Hero *Environment::getHero(void)
{
	return hero;
}

// --------------------------------------------------------

inline
void Environment::destroyBlock(const GLPoint& position)
{
	destroyBlock(position.x, position.y);
}

// --------------------------------------------------------

inline
const GLPoint& Environment::getMapOffset(void) const
{
	return mapOffset;
}

// --------------------------------------------------------

inline
const SDL_Rect& Environment::getMotionLessZone(void) const
{
	return motionLessScreen;
}

// --------------------------------------------------------

#endif	// _ENVIRONMENT_
