
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "Hero.hpp"

#define min(a,b)	(((a) <= (b))? (a) : (b))
#define max(a,b)	(((a) >= (b))? (a) : (b))

// --------------------------------------------------------

Hero::Hero(Environment *env, MapManager* aMap,SDL_Surface *aSprite,int aSpeed,int aLifeCount)
:Character(env, aMap,aSprite,aSpeed)
{
	// place itself on the entry point
	setPosition(map->getEntryPoint().x, map->getEntryPoint().y);
	lifeCount = aLifeCount;
	keysCount = 0;
}

// --------------------------------------------------------

Hero::~Hero()
{
}

// --------------------------------------------------------

void Hero::exitScreen(void)
{
	Character::exitScreen();

	lifeCount--;
}

// --------------------------------------------------------

void Hero::crush(void)
{
	Character::crush();

	lifeCount--;
}

// --------------------------------------------------------

void Hero::collide(void)
{
	initialize();

	lifeCount--;
}

// --------------------------------------------------------

bool Hero::finishLevel(void)
{
	return (map->isExitPoint(position) && (keysCount == map->getKeysCount()));
}

// --------------------------------------------------------

void Hero::newLevel(void)
{
	initialize();

	lifeCount++;
	keysCount = 0;

	checkCurrentPosition();
}

// --------------------------------------------------------

void Hero::checkCurrentPosition(void)
{
	if (map->isKey(position))
	{
		keysCount++;
		// if key, update map info
		map->deleteItem(position);
	}

	Character::checkCurrentPosition();
}

// --------------------------------------------------------

void Hero::putHoleRight(void)
{
	GLPoint holePosition = position + 1;

	// put a hole out of the screen
	if (!map->isInMap(holePosition))
	{
		return;
	}
	// put a hole where there is no ground or if wall on the right
	if ((!map->isGround(holePosition)) || (map->isGround(position.x+1,position.y)))
	{
		return;
	}
	
	environment->destroyBlock(holePosition);
	//SDL_Delay(500);
}

// --------------------------------------------------------

void Hero::putHoleLeft(void)
{
	GLPoint holePosition(position.x-1, position.y+1);

	// put a hole out of the screen
	if (!map->isInMap(holePosition))
	{
		return;
	}
	// put a hole where there is no ground or if wall on the right
	if ((!map->isGround(holePosition)) || (map->isGround(position.x-1,position.y)))
	{
		return;
	}
	environment->destroyBlock(holePosition);
	//SDL_Delay(500);
}

// --------------------------------------------------------

void Hero::initialize(void)
{
	Character::initialize();
	setPosition(map->getEntryPoint().x, map->getEntryPoint().y);
}

// --------------------------------------------------------

void Hero::setPosition(int x, int y)
{
	Character::setPosition(x, y);

	GLPoint pix = getPixelPosition();
	const SDL_Rect& mlz = environment->getMotionLessZone();
	GLPoint mapOff = environment->getMapOffset();

	if(pix.x - mapOff.x > mlz.x + mlz.w)
		mapOff.x = min(pix.x - (mlz.x + mlz.w), map->getPixelSize().x - environment->getScreenSize().x);
	else if(pix.x - environment->getMapOffset().x < mlz.x)
		mapOff.x = max(0, pix.x - mlz.x);

	if(pix.y - mapOff.y > mlz.y + mlz.h)
		mapOff.y = min(pix.y - (mlz.y + mlz.h), map->getPixelSize().y - environment->getScreenSize().y);
	else if(pix.y - mapOff.y < mlz.y)
		mapOff.y = max(0, pix.y - mlz.y);

	environment->setMapOffset(mapOff.x, mapOff.y);
}

// --------------------------------------------------------
