
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "OptionsManager.hpp"

#include "toollib/TLFiles.hpp"

#include <io.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// --------------------------------------------------------
// static variables

OptionsManager OptionsManager::defaultManager;

// --------------------------------------------------------

OptionsManager::OptionsManager(void)
{
	videoMode.width = _OPTIONS_DEFAULT_VIDEO_WIDTH;
	videoMode.height = _OPTIONS_DEFAULT_VIDEO_HEIGHT;
	videoMode.depth = _OPTIONS_DEFAULT_VIDEO_DEPTH;
	videoMode.mode = _OPTIONS_DEFAULT_VIDEO_MODE;

	folders.images = _OPTIONS_DEFAULT_FOLDERS_IMAGES;
	folders.maps = _OPTIONS_DEFAULT_FOLDERS_MAPS;

	game.startLevel = _OPTIONS_DEFAULT_GAME_STARTLEVEL;
}

// --------------------------------------------------------

OptionsManager::~OptionsManager()
{
	if(folders.images != NULL)
		delete[] (char *)(folders.images);
	if(folders.maps!= NULL)
		delete[] (char *)(folders.maps);
	if(game.startLevel!= NULL)
		delete[] (char *)(game.startLevel);
}

// --------------------------------------------------------

bool OptionsManager::load(const char *path)
{
	char line[1025];
	int handle;
	char *valuePointer;

	handle = open(path, O_RDONLY|O_TEXT);
	if(handle<0)
		return false;

	while(TLFiles::readLine(handle, line, 1024))
	{
		trim(line);
		if((strlen(line) > 0) && (strncmp(line, "//", 2) != 0))
		{
			valuePointer = strchr(line, '=');
			if(valuePointer == NULL)
				fprintf(stderr,"[error] unable to treat line : <%s>... continue...\n", line);
			else
			{
				*valuePointer = '\0';
				valuePointer++;
				trim(line);
				trim(valuePointer);
				setOption(line, valuePointer);
			}
		}
		//else	// commentary, we do nothing...
	}

	close(handle);

	return true;
}

// --------------------------------------------------------

char *OptionsManager::trim(char *string)
{
	int i = 0, count = strlen(string);

	while((i < count) && (string[i] <= ' '))
		i++;
	if(i > 0)
	{
		for(int j = 0; j < count-i; j++)
			string[j] = string[i+j];
		count -=i;
		string[count] = '\0';
	}
	i = count - 1;
	while((i>=0) && (string[i] <= ' '))
		i--;
	if(i < count - 1)
		string[i+1] = '\0';

	return string;
}

// --------------------------------------------------------

bool OptionsManager::setOption(const char *key, const char *value)
{
	printf("[infor] Options setted : <%s> = <%s>.\n", key, value);

	if(strcmp(key, _OPTIONS_KEY_VIDEO_WIDTH) == 0)
	{
		videoMode.width = atoi(value);
		if(videoMode.width == 0)
		{
			videoMode.width = _OPTIONS_DEFAULT_VIDEO_WIDTH;
			fprintf(stderr, "[error] Bad video width in configuration file <%s>. Value setted to <%d>.\n", value, _OPTIONS_DEFAULT_VIDEO_WIDTH);
			return false;
		}
	}
	else if(strcmp(key, _OPTIONS_KEY_VIDEO_HEIGHT) == 0)
	{
		videoMode.height = atoi(value);
		if(videoMode.height == 0)
		{
			videoMode.height = _OPTIONS_DEFAULT_VIDEO_HEIGHT;
			fprintf(stderr, "[error] Bad video height in configuration file <%s>. Value setted to <%d>.\n", value, _OPTIONS_DEFAULT_VIDEO_HEIGHT);
			return false;
		}
	}
	else if(strcmp(key, _OPTIONS_KEY_VIDEO_DEPTH) == 0)
	{
		videoMode.depth = atoi(value);
		if(videoMode.depth == 0)
		{
			videoMode.depth = _OPTIONS_DEFAULT_VIDEO_DEPTH;
			fprintf(stderr, "[error] Bad video depth in configuration file <%s>. Value setted to <%d>.\n", value, _OPTIONS_DEFAULT_VIDEO_DEPTH);
			return false;
		}
	}
	else if(strcmp(key, _OPTIONS_KEY_VIDEO_MODE) == 0)
	{
		if(strcmp(value, _OPTIONS_VALUE_VIDEO_MODE_WINDOW) == 0)
			videoMode.mode = wmWindowed;
		else if(strcmp(value, _OPTIONS_VALUE_VIDEO_MODE_FULLSCREEN) == 0)
			videoMode.mode = wmFullScreen;
		else
		{
			videoMode.mode = _OPTIONS_DEFAULT_VIDEO_MODE;
			fprintf(stderr, "[error] Bad video mode in configuration file <%s>. Value setted to <%s>.\n", value, (_OPTIONS_DEFAULT_VIDEO_MODE == wmWindowed)? _OPTIONS_VALUE_VIDEO_MODE_WINDOW : _OPTIONS_VALUE_VIDEO_MODE_FULLSCREEN);
			return false;
		}
	}
	else if(strcmp(key, _OPTIONS_KEY_FOLDERS_IMAGES) == 0)
	{
		int count;

		if(folders.images != NULL)
			delete[] (char *)(folders.images);
		count = strlen(value);
		if(count > 0)
		{
			folders.images = new char[count+1];
			strcpy(((char *)(folders.images)), value);
		}
		else
			folders.images = NULL;
	}
	else if(strcmp(key, _OPTIONS_KEY_FOLDERS_MAPS) == 0)
	{
		int count;

		if(folders.maps != NULL)
			delete[] (char *)(folders.maps);
		count = strlen(value);
		if(count > 0)
		{
			folders.maps = new char[count+1];
			strcpy(((char *)(folders.maps)), value);
		}
		else
			folders.images = NULL;
	}
	else if(strcmp(key, _OPTIONS_KEY_GAME_STARTLEVEL) == 0)
	{
		int count;

		if(game.startLevel != NULL)
			delete[] (char *)(game.startLevel);
		count = strlen(value);
		if(count > 0)
		{
			game.startLevel = new char[count+1];
			strcpy(((char *)(game.startLevel)), value);
		}
		else
			game.startLevel = NULL;
	}
	else
	{
		fprintf(stderr, "[error] Bad key in configuration file <%s>.\n", key);
		return false;
	}

	return true;
}

// --------------------------------------------------------
