
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                Vincent Pelletier <vpel@mageos.com>                     **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _OPTIONSMANAGER_
#define _OPTIONSMANAGER_

// --------------------------------------------------------
// --- video
#define _OPTIONS_DEFAULT_VIDEO_WIDTH	800
#define _OPTIONS_DEFAULT_VIDEO_HEIGHT	600
#define _OPTIONS_DEFAULT_VIDEO_DEPTH	 32
#define _OPTIONS_DEFAULT_VIDEO_MODE		wmWindowed

#define _OPTIONS_KEY_VIDEO_WIDTH		"video.width"
#define _OPTIONS_KEY_VIDEO_HEIGHT		"video.height"
#define _OPTIONS_KEY_VIDEO_DEPTH		"video.depth"
#define _OPTIONS_KEY_VIDEO_MODE			"video.mode"

#define _OPTIONS_VALUE_VIDEO_MODE_WINDOW		"window"
#define _OPTIONS_VALUE_VIDEO_MODE_FULLSCREEN	"fullscreen"

#define _OPTIONS_DEFAULT_VIDEO_WIDTH	800
#define _OPTIONS_DEFAULT_VIDEO_HEIGHT	600

// --- folders
#define _OPTIONS_DEFAULT_FOLDERS_IMAGES	NULL
#define _OPTIONS_DEFAULT_FOLDERS_MAPS	NULL

#define _OPTIONS_KEY_FOLDERS_IMAGES		"folders.images"
#define _OPTIONS_KEY_FOLDERS_MAPS		"folders.maps"

// --- game
#define _OPTIONS_DEFAULT_GAME_STARTLEVEL	NULL

#define _OPTIONS_KEY_GAME_STARTLEVEL		"game.startlevel"

// --------------------------------------------------------

class OptionsManager
{
	public:
		typedef enum {wmWindowed, wmFullScreen} EWindowMode;
		typedef struct 
		{
			int width;
			int height;
			int depth;
			EWindowMode mode;
		} VideoMode;

		typedef struct
		{
			const char *images;
			const char *maps;
		} Folders;

		typedef struct
		{
			const char *startLevel;
		} Game;

		static OptionsManager defaultManager;
	private:
		VideoMode videoMode;
		Folders folders;
		Game game;

		OptionsManager(void);
	public:
		virtual ~OptionsManager();

		bool load(const char *path);

		VideoMode getVideoMode(void) const;
		Folders getFolders(void) const;
		Game getGameParameters(void) const;

		bool setOption(const char *key, const char *value);

	protected:
		char *trim(char *string);

};

// --------------------------------------------------------
// inline methods

inline
OptionsManager::VideoMode OptionsManager::getVideoMode(void) const
{
	return videoMode;
}

// --------------------------------------------------------

inline
OptionsManager::Folders OptionsManager::getFolders(void) const
{
	return folders;
}

// --------------------------------------------------------

inline
OptionsManager::Game OptionsManager::getGameParameters(void) const
{
	return game;
}

// --------------------------------------------------------

#endif // _OPTIONSMANAGER_

