
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "ResourcePool.hpp"

#include <string.h>
#include <SDL_image.h>

// --------------------------------------------------------
// static variables

ResourcePool ResourcePool::defaultPool;

static const char *imageTypeString[] = {".bmp"};

// --------------------------------------------------------

ResourcePool::ResourcePool(void)
: imagePool(true, false)
{
	imageFolder = NULL;

	for(int i=0, count= imagePool.getCount(); i<count; i++)
	{
		SDL_FreeSurface((SDL_Surface *)(imagePool.getValue(i)));
	}
}	

// --------------------------------------------------------

ResourcePool::~ResourcePool()
{
	if(imageFolder != NULL)
		delete[] imageFolder;
}

// --------------------------------------------------------

/**
 * return -1 if image can't be loaded... else return the index of image in the pool.
 * 
 * @todo : introduce a notion of log to replace printf("[infor]"...)
 */
SDL_Surface *ResourcePool::loadImage(const char *name, EImageType type)
{
	SDL_Surface *image;
	char *filePath;

	image = getImage(name);
	if(image != NULL)
		return image;

printf("[infor] ResourcePool::loadImage(%s)\n", name);
	if(imageFolder != NULL)
	{
		filePath = new char[strlen(imageFolder)+strlen(name)+strlen(imageTypeString[type])+1];
		strcpy(filePath, imageFolder);
		strcat(filePath, name);
		strcat(filePath, imageTypeString[type]);
	}
	else
	{
		filePath = new char[strlen(name)+strlen(imageTypeString[type])+1];
		strcpy(filePath, name);
		strcat(filePath, imageTypeString[type]);
	}

	image = SDL_LoadBMP(filePath);
	if(image != NULL)
		return (SDL_Surface *)imagePool.add(name,image);
	else
	{
		fprintf(stderr, "ERROR : Unable to load image : <%s>.\n", filePath);
		exit(-2);
	}

	return image;
}

// --------------------------------------------------------

SDL_Surface *ResourcePool::getImage(const char *name)
{
	return (SDL_Surface *)(imagePool.get(name));
}

// --------------------------------------------------------

void ResourcePool::setImageFolder(const char *folder)
{
	int len;
	bool addSlash;

	if(imageFolder != NULL)
	{
		delete[] imageFolder;
		imageFolder = NULL;
	}

	if(folder != NULL)
	{ 
		len = strlen(folder);
		if(len > 0)
		{
			addSlash = ((folder[len - 1] != '\\') && (folder[len - 1] != '/'));
				
			imageFolder = new char[len+1+(addSlash?1:0)];
			strcpy(imageFolder, folder);
			if(addSlash)
				strcat(imageFolder, "/");
		}
	}
}

// --------------------------------------------------------
