
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "GLFont.hpp"

#include <string.h>

// --------------------------------------------------------

static char *letterOrder = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.?!0123456789,;:()'\"-& ";

// --------------------------------------------------------

GLFont::GLFont(void)
{
	image = NULL;
}

// --------------------------------------------------------

GLFont::GLFont(const char *imgFile, int w, int h, int columns, int rows)
{
	letterSize.set(w, h);
	fontSize.set(columns, rows);
	image = SDL_LoadBMP(imgFile);
	if(image != NULL)
		SDL_SetColorKey(image,SDL_SRCCOLORKEY,SDL_MapRGB(image->format, 0, 0, 255));
}

// --------------------------------------------------------

GLFont::~GLFont()
{
	if(image != NULL)
		SDL_FreeSurface(image);
}

// --------------------------------------------------------

void GLFont::writeText(const GLPoint& position, const char *text, SDL_Surface *surface)
{
	int len = strlen(text);
	GLPoint pos(position);

	for(int i= 0; i<len; i++)
	{
		if(text[i] == '\n')
			pos.set(position.x, pos.y + letterSize.y);
		else
		{
			if(writeCharacter(pos, text[i], surface))
				pos.x += letterSize.x;
		}
	}
}
		
// --------------------------------------------------------
bool GLFont::writeCharacter(const GLPoint& position, char letter, SDL_Surface *surface)
{
	SDL_Rect rect;
	SDL_Rect destination;

	if(getCharacterRect(letter, &rect))
	{
		destination.x = position.x;
		destination.y = position.y;
		if(image != NULL)
			SDL_BlitSurface(image,&rect,surface,&destination);
		return true;
	}
	else
		return false;
}

// --------------------------------------------------------

bool GLFont::getCharacterRect(char letter, SDL_Rect *rect)
{
	int len = strlen(letterOrder);

	for(int i=0; i<len; i++)
	{
		if(letterOrder[i] == letter)
		{
			rect->x = (i % fontSize.x) * letterSize.x;
			rect->y = (i / fontSize.x) * letterSize.y;
			rect->w = letterSize.x;
			rect->h = letterSize.y;
			return true;
		}
	}
 
	return false;
}

// --------------------------------------------------------

GLPoint GLFont::getTextSize(const char *text) const
{
	int len = strlen(text);
	GLPoint pos;
	int maxWidth = 0;

	if(len == 0)
		return GLPoint(0,0);

	for(int i= 0; i<len; i++)
	{
		if(text[i] == '\n')
		{
			if(pos.x > maxWidth)
				maxWidth = pos.x;
			pos.set(0, pos.y + letterSize.y);
		}
		else
		{
			pos.x += letterSize.x;
		}
	}

	if(pos.x  > maxWidth)
		maxWidth = pos.x;

	return GLPoint(maxWidth, pos.y + letterSize.y);
}

// --------------------------------------------------------
