
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _GLPOINT_
#define _GLPOINT_


// --------------------------------------------------------

/**
 * Graphical Point 
 *
 */
class GLPoint
{
	public:
		int x;
		int y;

	public:
		GLPoint(void);
		GLPoint(int aX, int aY);
		GLPoint(const GLPoint& other);
		virtual ~GLPoint();

		void set(int aX, int aY);

		GLPoint operator+(int offset) const;
		GLPoint operator+(const GLPoint& other) const;
		GLPoint operator-(int offset) const;
		GLPoint operator-(const GLPoint& other) const;
		GLPoint operator*(int offset) const;
		GLPoint operator/(int offset) const;

		bool operator==(const GLPoint& other) const;
		bool operator!=(const GLPoint& other) const;

		GLPoint& operator+=(int offset);
		GLPoint& operator+=(const GLPoint& other);
		GLPoint& operator-=(int offset);
		GLPoint& operator-=(const GLPoint& other);
		GLPoint& operator*=(int offset);
		GLPoint& operator/=(int offset);
};

// --------------------------------------------------------
// inline methods

inline
GLPoint::GLPoint(void)
{
	x = y = 0;
}

// --------------------------------------------------------

inline
GLPoint::GLPoint(int aX, int aY)
{
	x = aX;
	y = aY;
}

// --------------------------------------------------------

inline
GLPoint::GLPoint(const GLPoint& other)
{
	x = other.x;
	y = other.y;
}

// --------------------------------------------------------

inline
GLPoint::~GLPoint()
{
	// nothing to do
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator+(int offset) const
{
	return GLPoint(x+offset, y+offset);
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator+(const GLPoint& other) const
{
	return GLPoint(x+other.x, y+other.y);
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator-(int offset) const
{
	return GLPoint(x-offset, y-offset);
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator-(const GLPoint& other) const
{
	return GLPoint(x-other.x, y-other.y);
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator*(int offset) const
{
	return GLPoint(x*offset, y*offset);
}

// --------------------------------------------------------

inline
GLPoint GLPoint::operator/(int offset) const
{
	return GLPoint(x/offset, y/offset);
}

// --------------------------------------------------------

inline
bool GLPoint::operator==(const GLPoint& other) const
{
	return (x == other.x) && (y == other.y);
}

// --------------------------------------------------------

inline
bool GLPoint::operator!=(const GLPoint& other) const
{
	return (x != other.x) || (y != other.y);
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator+=(int offset) 
{
	x += offset;
	y += offset;

	return *this;
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator+=(const GLPoint& other) 
{
	x += other.x;
	y -= other.y;

	return *this;
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator-=(int offset) 
{
	x -= offset;
	y -= offset;

	return *this;
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator-=(const GLPoint& other) 
{
	x -= other.x;
	y -= other.y;

	return *this;
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator*=(int offset) 
{
	x *= offset;
	y *= offset;

	return *this;
}

// --------------------------------------------------------

inline
GLPoint& GLPoint::operator/=(int offset) 
{
	x /= offset;
	y /= offset;

	return *this;
}

// --------------------------------------------------------

inline
void GLPoint::set(int aX, int aY)
{
	x = aX;
	y = aY;
}

// --------------------------------------------------------

#endif	// _GLPOINT_
