
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _TLCOUPLE_
#define _TLCOUPLE_

#include <stdlib.h>

// --------------------------------------------------------

/**
 * generic couple (key, value)
 *
 */
class TLCouple
{
	private:
		const void *key;
		const void *value;

		bool destroy;
	public:
		TLCouple(bool destroyFlag = false);
		TLCouple(const void *aKey, const void *aValue, bool destroyFlag = false);
		virtual ~TLCouple();

		void setDestructor(bool destroyFlag);
		bool isDestructor(void) const;

		void setKey(const void *aKey);
		const void *getKey(void) const;

		void setValue(const void *aValue);
		const void *getValue(void) const;

		void set(const void *aKey, const void *aValue);
	protected:
};

// --------------------------------------------------------
// inline methods

inline
TLCouple::TLCouple(bool destroyFlag)
{
	key = NULL;
	value = NULL;
	destroy = destroyFlag;
}

// --------------------------------------------------------

inline
TLCouple::TLCouple(const void *aKey, const void *aValue, bool destroyFlag)
{
	key = aKey;
	value = aValue;
	destroy = destroyFlag;
}

// --------------------------------------------------------

inline
TLCouple::~TLCouple()
{
	if(destroy)
	{
		if(key != NULL)
			delete (void *)key;
		if(value != NULL)
			delete (void *)value;
	}
}

// --------------------------------------------------------

inline
void TLCouple::setDestructor(bool destroyFlag)
{
	destroy = destroyFlag;
}

// --------------------------------------------------------

inline
bool TLCouple::isDestructor(void) const
{
	return destroy;
}

// --------------------------------------------------------

inline
void TLCouple::setKey(const void *aKey)
{
	if((destroy) && (key != NULL))
		delete (void *)key;
	key = aKey;
}

// --------------------------------------------------------

inline
const void *TLCouple::getKey(void) const
{
	return key;
}

// --------------------------------------------------------

inline
void TLCouple::setValue(const void *aValue)
{
	if((destroy) && (value != NULL))
		delete (void *)value;
	value = aValue;
}

// --------------------------------------------------------

inline
const void *TLCouple::getValue(void) const
{
	return value;
}

// --------------------------------------------------------

inline
void TLCouple::set(const void *aKey, const void *aValue)
{
	if(destroy)
	{
		if(key != NULL)
			delete (void *)key;
		if(value != NULL)
			delete (void *)value;
	}

	key = aKey;
	value = aValue;
}

// --------------------------------------------------------

#endif	// _TLCOUPLE_


