
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#include "TLDictionary.hpp"

// --------------------------------------------------------

TLDictionary::TLDictionary(bool destroyFlag, bool replaceFlag)
: couples(true)
{
	destroy = destroyFlag;
	replace = replaceFlag;
	equalityFunction = NULL;
}

// --------------------------------------------------------

TLDictionary::TLDictionary(EqualityFunction function, bool destroyFlag, bool replaceFlag)
{
	destroy = destroyFlag;
	replace = replaceFlag;
	equalityFunction = function;
}

// --------------------------------------------------------

TLDictionary::~TLDictionary()
{
}

// --------------------------------------------------------

/**
 * add or replace a couple into dictionary.
 */
const void *TLDictionary::add(const void *key, const void *value)
{
	if(equalityFunction != NULL)
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(equalityFunction(((TLCouple *)couples[i])->getKey(), key))
			{
				if(replace)
				{
					((TLCouple *)couples[i])->setValue(value);
					return value;
				}
				else
					return ((TLCouple *)couples[i])->getValue();
			}
		}
	}
	else
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(((TLCouple *)couples[i])->getKey() == key)
			{
				if(replace)
				{
					((TLCouple *)couples[i])->setValue(value);
					return value;
				}
				else
					return ((TLCouple *)couples[i])->getValue();
			}
		}
	}

	couples.add(new TLCouple(key, value));
	return value;
}

// --------------------------------------------------------

void TLDictionary::remove(const void *key)
{
	if(equalityFunction != NULL)
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(equalityFunction(((TLCouple *)couples[i])->getKey(), key))
			{
				couples.remove(i);
				return;
			}
		}
	}
	else
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(((TLCouple *)couples[i])->getKey() == key)
			{
				couples.remove(i);
				return;
			}
		}
	}
}

// --------------------------------------------------------

void TLDictionary::clear(void)
{
	couples.clear();
}

// --------------------------------------------------------

const void *TLDictionary::get(const void *key) const
{
	if(equalityFunction != NULL)
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(equalityFunction(((TLCouple *)couples[i])->getKey(), key))
				return ((TLCouple *)couples[i])->getValue();
		}
	}
	else
	{
		for(int i=0, count = couples.getCount(); i<count; i++)
		{
			if(((TLCouple *)couples[i])->getKey() == key)
				return ((TLCouple *)couples[i])->getValue();
		}
	}

	return NULL;
}

// --------------------------------------------------------
