
/***************************************************************************
**                         BIBOM - LODERUNNER                             **
**                      Copyright (C) 2002-2003,                          **
**                                                                        **
**  This program is free software; you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation; either version 2 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
***************************************************************************/ 

#ifndef _TLDICTIONARY_
#define _TLDICTIONARY_

#include "TLArray.hpp"
#include "TLCouple.hpp"

// --------------------------------------------------------

/**
 * Dynamic dictionary
 *
 */
class TLDictionary
{
	public:
		typedef bool (*EqualityFunction)(const void *, const void *);

	private:
		TLArray couples;

		bool destroy;
		bool replace;
		
		EqualityFunction equalityFunction;
	public:
		TLDictionary(bool destroyFlag = false, bool replaceFlag = false);
		TLDictionary(EqualityFunction function, bool destroyFlag = false, bool replaceFlag = false);
		virtual ~TLDictionary();

		const void *add(const void *key, const void *value);
		void remove(const void *key);
		void clear(void);

		const void *get(const void *key) const;

		void setDestructor(bool destroyFlag);
		void setReplacing(bool replaceFlag);
		int getCount(void) const;
		const void *getKey(int index) const;
		const void *getValue(int index) const;
};

// --------------------------------------------------------
// inline methods

inline
void TLDictionary::setDestructor(bool destroyFlag)
{
	destroy = destroyFlag;

	for(int i = 0, count = couples.getCount(); i< count; i++)
		((TLCouple *)couples[i])->setDestructor(destroyFlag);
}

// --------------------------------------------------------

inline
void TLDictionary::setReplacing(bool replaceFlag)
{
	replace = replaceFlag;
}

// --------------------------------------------------------
inline
int TLDictionary::getCount(void) const
{
	return couples.getCount();
}

// --------------------------------------------------------

inline
const void *TLDictionary::getKey(int index) const
{
	if((index>=0) && (index<getCount()))
		return ((TLCouple *)couples[index])->getKey();
	else
		return NULL;
}

// --------------------------------------------------------

inline
const void *TLDictionary::getValue(int index) const
{
	if((index>=0) && (index<getCount()))
		return ((TLCouple *)couples[index])->getValue();
	else return NULL;
}

// --------------------------------------------------------

#endif	// _TLDICTIONARY_


